/*
 * kaddressbook.cpp
 *
 * Copyright (C) 1999 Don Sanders <dsanders@kde.org>
 */
#include "kaddressbook.h"

#include <qkeycode.h>
#include <qregexp.h>
#include <qlayout.h>

#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kstandarddirs.h>
#include <kprocess.h>
#include <kapplication.h>
#include <kprotocolinfo.h>
#include <kabc/stdaddressbook.h>
#include <kabc/distributionlisteditor.h>
#include <kabc/distributionlist.h>
#include <kabc/vcard21parser.h>

#include "undo.h"
#include "undocmds.h"
#include "contactentry.h"
#include "contactentrylist.h"
#include "contactlistview.h"
#include "browserentryeditor.h"
#include "importdialog.h"
#include "prefsdialog.h"
#include "ldapsearchdialogimpl.h"
#include "viewmanager.h"

KAddressBook::KAddressBook( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  QBoxLayout *topLayout = new QVBoxLayout( this );

  mDocument = new ContactEntryList();
  
  // Create the GUI
  mViewManager = new ViewManager(mDocument, kapp->config(), 
                                 this, "mViewManager");
  topLayout->addWidget(mViewManager);
  connect(mViewManager, SIGNAL(selected(const QString &)),
          this, SLOT(addresseeSelected(const QString &)));
  connect(mViewManager, SIGNAL(executed(const QString &)),
          this, SLOT(addresseeExecuted(const QString &)));
  connect(mViewManager, SIGNAL(modified()), this,
          SLOT(viewModified()));
  
  mDistEditor = 0;
  mPrefsDialog = 0;
  mLdapSearchDialog = 0;
}


void KAddressBook::newContact()
{
    kdDebug() << "KAddressBook::newContact() start" << endl;
    ContactDialog *cd = new PabNewContactDialog(i18n("Address Book Entry Editor"), 
                                              this, 0 );

    // This method needs to be updated to the new view design.
    //  connect( cd, SIGNAL( add( ContactEntry* ) ),
    //	   mView, SLOT( addNewEntry( ContactEntry* ) ));
    cd->show();
    kdDebug() << "KAddressBook::newContact() stop" << endl;
}

void KAddressBook::slotDistributionList()
{
  save();
  if (!mDistEditor)
      mDistEditor = new KABC::DistributionListEditor(
	  KABC::StdAddressBook::self(), 0 );

  mDistEditor->show();
}

void KAddressBook::addEmail( QString aStr )
{
  int i, j, len;
  QString partA, partB, result;
  char endCh = '>';

  i = aStr.find('<');
  if (i<0)
  {
    i = aStr.find('(');
    endCh = ')';
  }
  if (i<0) {
    updateContact( aStr, "" );
    return;
  }
  partA = aStr.left(i).stripWhiteSpace();
  j = aStr.find(endCh,i+1);
  if (j<0) {
    updateContact( aStr, "" );
    return;
  }
  partB = aStr.mid(i+1, j-i-1).stripWhiteSpace();

  if (partA.find('@') >= 0 && !partB.isEmpty()) result = partB;
  else if (!partA.isEmpty()) result = partA;
  else result = aStr;

  len = result.length();
  if (result[0]=='"' && result[len-1]=='"')
    result = result.mid(1, result.length()-2);
  else if (result[0]=='<' && result[len-1]=='>')
    result = result.mid(1, result.length()-2);
  else if (result[0]=='(' && result[len-1]==')')
    result = result.mid(1, result.length()-2);

  updateContact( partB, result );
}

ASYNC KAddressBook::showContactEditor ( QString uid )
{
  editAddressee(uid);
}

void KAddressBook::addEntry(ContactEntry newEntry)
{
  PwNewCommand *command = new PwNewCommand(mDocument, 
                                           new ContactEntry(newEntry));
  UndoStack::instance()->push( command );
  RedoStack::instance()->clear();
  
  mViewManager->refresh();
}

QStringList KAddressBook::getKeys() const
{
  return mDocument->keys();
}

QDict<ContactEntry> KAddressBook::getEntryDict() const
{
  return mDocument->getDict();
}

void  KAddressBook::changeEntry( QString key, ContactEntry changeEntry)
{
  mDocument->replace(key, new ContactEntry(changeEntry));
  mViewManager->refresh(key);
}

void  KAddressBook::removeEntry( QString key )
{
  mDocument->remove(key);
  mViewManager->refresh();
}

void KAddressBook::editAddressee(QString uid)
{
    // This method needs to be updated when the API for the edit
    // addressee dialog is updated. Currently things are very messy
    // with the edit dialogs and the contactEntryList stuff. -mpilone
    
    ContactEntry *ce;

    // First, locate the contact entry
    if (uid == QString::null)
    {
        QStringList uidList = mViewManager->selectedUids();
        if (uidList.count() > 0)
            uid = *(uidList.at(0));
    }
   
    ce = mDocument->find(uid);
    
    // This should never fail, unless someone gives us a bogus uid!
    Q_ASSERT(ce);
    
    // Now we have a valid contact, edit away
    PabContactDialog dialog(i18n("Edit Addressee"), this, "PabContactDialog",
                            uid, ce, true);
    
    if (dialog.exec())
    {
        // Update the contact in the document. There is no need to refresh
        // the view. That will be done in the update.
        updateAddressee(uid, dialog.entry());
 
        // The user accepted so we need to let listeners know that
        // we were modified.
        emit modified(true);
    }
}

void KAddressBook::newAddressee()
{
    kdDebug() << "KAddressBook::newAddressee()" << endl;

    // Create the dialog
    PabNewContactDialog dialog(i18n("Edit Addressee"), this, 
                               "PabContactDialog", true);
    
    if (dialog.exec())
    {
        // Update the contact in the document. No need to refresh the view,
        // that will be done in the update.
        updateAddressee("", dialog.entry());
 
        // The user accepted so we need to let listeners know that
        // we were modified.
        emit modified(true);
    }
}

void KAddressBook::save()
{
//  mDistEditor->manager()->save(); need cornelius help
  mDocument->commit();
  mDocument->refresh();
  
  emit modified(false);
}

void KAddressBook::readConfig()
{
  mViewManager->readConfig();
}

void KAddressBook::writeConfig()
{
  mViewManager->writeConfig();
}

KAddressBook::~KAddressBook()
{
  if (mDistEditor)
      delete mDistEditor; // save distribution lists
      
  writeConfig();
  
  delete mDocument;
}

void KAddressBook::undo()
{
  kdDebug() << "KAddressBook::undo()" << endl;
  UndoStack::instance()->undo();
  
  // Refresh the view
  mViewManager->refresh();
}

void KAddressBook::redo()
{
  RedoStack::instance()->redo();
  
  // Refresh the view
  mViewManager->refresh();
}

void KAddressBook::importKDE2()
{
  if (!QFile::exists(locateLocal("data", "kabc/std.vcf") )) {
    KMessageBox::sorry( this, i18n("Couldn't find a KDE 2 address book.") );
    return;
  }

  int result = KMessageBox::questionYesNoCancel( this,
      i18n("Override previously imported entries?"),
      i18n("Import KDE 2 addressbook"));

  if ( !result ) return;

  KProcess proc;

  if ( result == KMessageBox::Yes ) {
    proc << "kab2kabc";
    proc << "--override";
  } else if ( result == KMessageBox::No ) {
    proc << "kab2kabc";
  } else {
    kdDebug() << "KAddressBook::importKDE2(): Unknow return value." << endl;
  }

  proc.start( KProcess::Block );

  mDocument->refresh();
  mViewManager->refresh();
}

void KAddressBook::importCSV()
{
  kdDebug() << "KAddressBook::importCSV()" << endl;

  ContactImportDialog *dialog = new ContactImportDialog(mDocument,this);

  dialog->exec();

  mDocument->refresh();
  mViewManager->refresh();

  delete dialog;
}

void KAddressBook::importVCard()
{
  ContactEntryList *list = new ContactEntryList();
  QString fileName = KFileDialog::getOpenFileName(QString::null,
                                                  "*.vcf", 0,
                                                  i18n("Select vCard to open"));
                                                  
  if (!fileName.isEmpty())
  {
      KABC::VCard21Parser *parser = new KABC::VCard21Parser();
      KABC::Addressee a;
      QFile file(fileName);
      
      file.open( IO_ReadOnly );
      QByteArray rawData = file.readAll();
      QString data = QString::fromLatin1( rawData.data(), rawData.size() + 1 );
      a = parser->readFromString( data);
      
      if (!a.isEmpty()) 
      {
          ContactEntry *entry = list->KabEntryToContactEntry(a);
          delete parser;
          
          ContactDialog *dialog = new ContactDialog("Imported vCard", 0, "",
                                                    entry);
          dialog->show();
          
          // This needs to be updated once the dialog gets
          // a make over. -mpilone
          //connect(dialog, SIGNAL( accepted(ContactEntry* ) ),
          //        mView, SLOT( addNewEntry( ContactEntry* ) ));
      }
  }
}

void KAddressBook::exportCSV()
{
  QString fileName = KFileDialog::getSaveFileName("addressbook.csv");

  QFile outFile(fileName);
  if ( outFile.open(IO_WriteOnly) ) {    // file opened successfully
    QTextStream t( &outFile );        // use a text stream

    QStringList keys = mDocument->keys();
    QStringList::ConstIterator it = keys.begin();
    QStringList::ConstIterator end = keys.end();

    t << "\"First Name\",";
    t << "\"Last Name\",";
    t << "\"Middle Name\",";
    t << "\"Name Prefix\",";
    t << "\"Job Title\",";
    t << "\"Company\",";
    t << "\"Email\",";
    t << "\"Nickname\",";
    t << "\"Note\",";
    t << "\"Business Phone\",";
    t << "\"Home Phone\",";
    t << "\"Mobile Phone\",";
    t << "\"Home Fax\",";
    t << "\"Business Fax\",";
    t << "\"Pager\",";
    t << "\"Street Home Address\",";
    t << "\"City Home Address\",";
    t << "\"State Home Address\",";
    t << "\"Zip Home Address\",";
    t << "\"Country Home Address\",";
    t << "\"Street Business Address\",";
    t << "\"City Business Address\",";
    t << "\"State Business Address\",";
    t << "\"Zip Business Address\",";
    t << "\"Country Business Address\"";
    t << "\n";

    while(it != end) {
      ContactEntry *entry = mDocument->find(*it);

      t << "\"" << entry->getFirstName() << "\",";
      t << "\"" << entry->getLastName() << "\",";
      t << "\"" << entry->getMiddleName() << "\",";
      t << "\"" << entry->getNamePrefix() << "\",";
      t << "\"" << entry->getJobTitle() << "\",";
      t << "\"" << entry->getCompany() << "\",";
      t << "\"" << entry->getEmail() << "\",";
      t << "\"" << entry->getNickname() << "\",";
      t << "\"" << entry->getNote() << "\",";
      t << "\"" << entry->getBusinessPhone() << "\",";
      t << "\"" << entry->getHomePhone() << "\",";
      t << "\"" << entry->getMobilePhone() << "\",";
      t << "\"" << entry->getHomeFax() << "\",";
      t << "\"" << entry->getBusinessFax() << "\",";
      t << "\"" << entry->getPager() << "\",";
      ContactEntry::Address *address;
      address = entry->getHomeAddress();
      t << "\"" << address->getStreet() << "\",";
      t << "\"" << address->getCity() << "\",";
      t << "\"" << address->getState() << "\",";
      t << "\"" << address->getZip() << "\",";
      t << "\"" << address->getCountry() << "\",";
      address = entry->getBusinessAddress();
      t << "\"" << address->getStreet() << "\",";
      t << "\"" << address->getCity() << "\",";
      t << "\"" << address->getState() << "\",";
      t << "\"" << address->getZip() << "\",";
      t << "\"" << address->getCountry() << "\",";
      t << "\n";

      ++it;
    }

    outFile.close();
  }
}

QString KAddressBook::getNameByPhone( QString phone )
{
  QStringList keys = mDocument->keys();
  QStringList::ConstIterator it = keys.begin();
  QStringList::ConstIterator end = keys.end();

  QRegExp r("[/*/-]");

  bool bSuccess = false;
  QString ownerName = "";	
  do {
    ContactEntry *entry = mDocument->find(*it);
  	
    QString homePhone   = entry->getHomePhone();
    QString mobilePhone = entry->getMobilePhone();
    QString bizPhone    = entry->getBusinessPhone();
  	
    homePhone = homePhone.stripWhiteSpace();
    homePhone.replace( r, "" );
  	
    bizPhone = bizPhone.stripWhiteSpace();
    bizPhone.replace( r, "" );
  	
    mobilePhone = mobilePhone.stripWhiteSpace();
    mobilePhone.replace( r, "" );
  	
    if( (homePhone == phone) || (mobilePhone == phone) || (bizPhone ==phone) ){
      bSuccess = true;
      ownerName = entry->getFirstName() + QString(" ") + entry->getLastName();
    }
    ++it;
  } while( it!=end && !bSuccess );

  return ownerName;
}

void KAddressBook::addresseeSelected(const QString &uid)
{
    emit addresseeSelected(uid != QString::null);
}
    
void KAddressBook::addresseeExecuted(const QString &uid)
{
    // Display the editor dialog.
    kdDebug() << "KAddressBook::addresseeExecuted: editing contact" << endl;
    
    if (uid != QString::null)
        editAddressee(uid);
}
    
void KAddressBook::viewModified()
{
    emit modified(true);
}

void KAddressBook::updateAddressee(QString entryKey, ContactEntry *ce)
{
  ContactEntry *oldce = mDocument->find(entryKey);
  if (oldce) {
    PwEditCommand *command = new PwEditCommand( mDocument, entryKey, oldce, ce );
    UndoStack::instance()->push( command );
    RedoStack::instance()->clear();
    
    mViewManager->refresh(entryKey);
  }
  else {
    PwNewCommand *command = new PwNewCommand( mDocument, ce );
    UndoStack::instance()->push( command );
    RedoStack::instance()->clear();
    
    mViewManager->refresh();
    mViewManager->setSelected(entryKey, true);
  }
}

void KAddressBook::configure()
{
  if ( !mPrefsDialog ) {
    mPrefsDialog = new PrefsDialog( this );
  }
  
  mPrefsDialog->show();
  mPrefsDialog->raise();
}

void KAddressBook::slotOpenLDAPDialog()
{
  if( !mLdapSearchDialog ) 
    mLdapSearchDialog = new LDAPSearchDialogImpl(this);
  else 
    mLdapSearchDialog->rereadConfig();

  if( mLdapSearchDialog->isOK() )
    mLdapSearchDialog->exec();
}


void KAddressBook::updateContact( QString addr, QString name )
{
  QStringList keys = mDocument->keys();
  for ( QStringList::Iterator it = keys.begin(); it != keys.end(); ++it ) 
  {
    ContactEntry *ce = mDocument->find( *it );
    if (ce)
      if (ce->find("EMAIL") && 
         ((*ce->find("EMAIL")).stripWhiteSpace() == addr)) 
      {
	      if (!name.isEmpty())
	        ce->replace( "N", new QString( name ) );
	
        editAddressee(*it);
        
        /* Original code **
        QString title = i18n( "Address Book Entry Editor" );
        PabContactDialog *cd = new PabContactDialog( title, this, 0, *it, ce );
        QObject::connect(cd, SIGNAL( change( QString, ContactEntry* ) ),
			                  this, SLOT( change( QString, ContactEntry* ) ));
        cd->show();
        **/
        
        return;
      }
  }

  ContactDialog *cd = new PabNewContactDialog(i18n("Address Book Entry Editor"),
                                              this, 0, true);
  ContactEntry *ce = cd->entry();
  if (!name.isEmpty())
    ce->replace( ".AUXCONTACT-N", new QString(name) );
  ce->replace( "EMAIL", new QString( addr ) );
  //connect( cd, SIGNAL( add( ContactEntry* ) ),
	//   this, SLOT( addNewEntry( ContactEntry* ) ));
  cd->parseName();
  if (cd->exec())
  {
    // Update the contact in the document. There is no need to refresh
    // the view. That will be done in the update.
    updateAddressee("", cd->entry());
 
    // The user accepted so we need to let listeners know that
    // we were modified.
    emit modified(true);
  }
  
  delete cd;
}

#include "kaddressbook.moc"
